package cn.com.qmth.examcloud.core.basic.api.controller;

import java.util.List;
import java.util.Map;
import java.util.Set;

import org.apache.commons.collections.CollectionUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Sort;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import com.google.common.collect.Lists;
import com.google.common.collect.Maps;
import com.google.common.collect.Sets;

import cn.com.qmth.examcloud.commons.base.exception.StatusException;
import cn.com.qmth.examcloud.commons.base.util.RegExpUtil;
import cn.com.qmth.examcloud.commons.web.helpers.tree.EleTreeNode;
import cn.com.qmth.examcloud.commons.web.helpers.tree.TreeUtil;
import cn.com.qmth.examcloud.commons.web.security.bean.User;
import cn.com.qmth.examcloud.commons.web.support.ControllerSupport;
import cn.com.qmth.examcloud.core.basic.api.controller.bean.PrivilegeGroupBean;
import cn.com.qmth.examcloud.core.basic.api.controller.bean.PrivilegeInfo;
import cn.com.qmth.examcloud.core.basic.api.controller.bean.UpdateRolePrivilegeRelationsReq;
import cn.com.qmth.examcloud.core.basic.base.constants.BasicConsts;
import cn.com.qmth.examcloud.core.basic.dao.AppRepo;
import cn.com.qmth.examcloud.core.basic.dao.OrgRepo;
import cn.com.qmth.examcloud.core.basic.dao.PrivilegeGroupRepo;
import cn.com.qmth.examcloud.core.basic.dao.PrivilegeRepo;
import cn.com.qmth.examcloud.core.basic.dao.RolePrivilegeRelationRepo;
import cn.com.qmth.examcloud.core.basic.dao.RoleRepo;
import cn.com.qmth.examcloud.core.basic.dao.entity.AppEntity;
import cn.com.qmth.examcloud.core.basic.dao.entity.PrivilegeEntity;
import cn.com.qmth.examcloud.core.basic.dao.entity.PrivilegeGroupEntity;
import cn.com.qmth.examcloud.core.basic.dao.entity.RolePrivilegeRelationEntity;
import cn.com.qmth.examcloud.core.basic.service.RolePrivilegeService;
import io.swagger.annotations.ApiOperation;

/**
 * 角色权限
 *
 * @author WANGWEI
 * @date 2018年6月12日
 * @Copyright (c) 2018-? http://qmth.com.cn All Rights Reserved.
 */
@RestController
@RequestMapping("${app.api.root}/rolePrivilege")
public class RolePrivilegeController extends ControllerSupport {

	@Autowired
	AppRepo appRepo;

	@Autowired
	PrivilegeGroupRepo privilegeGroupRepo;

	@Autowired
	PrivilegeRepo privilegeRepo;

	@Autowired
	OrgRepo orgRepo;

	@Autowired
	RoleRepo roleRepo;

	@Autowired
	RolePrivilegeRelationRepo rolePrivilegeRelationRepo;

	@Autowired
	RolePrivilegeService rolePrivilegeService;

	@ApiOperation(value = "查询用户的权限树", notes = "")
	@PostMapping("getUserPrivileges")
	public List<PrivilegeInfo> getPrivileges(@RequestParam String groupCode,
			@RequestParam boolean full) {

		User accessUser = getAccessUser();
		Long rootOrgId = accessUser.getRootOrgId();

		PrivilegeGroupEntity privilegeGroup = privilegeGroupRepo.findByCode(groupCode);

		if (null == privilegeGroup) {
			throw new StatusException("B-002001", "groupCode is not existing");
		}

		List<Long> roleIdList = getAccessUserRoleIdList();

		List<RolePrivilegeRelationEntity> rolePrivRelationList = rolePrivilegeRelationRepo
				.findAllByRoleIdInAndRootOrgId(roleIdList, rootOrgId);

		Set<String> pIdSet = Sets.newHashSet();
		for (RolePrivilegeRelationEntity cur : rolePrivRelationList) {
			pIdSet.add(String.valueOf(cur.getPrivilegeId()));
		}

		List<PrivilegeEntity> privilegeList = privilegeRepo
				.findAllByGroupIdOrderByWeightDesc(privilegeGroup.getId());

		List<PrivilegeInfo> privilegeInfoList = Lists.newArrayList();

		for (PrivilegeEntity cur : privilegeList) {
			boolean hasPrivilege = pIdSet.contains(String.valueOf(cur.getId()));
			// 超级管理员拥有所有权限
			if (isSuperAdmin()) {
				hasPrivilege = true;
			}
			if ((!full) && (!hasPrivilege)) {
				continue;
			}

			PrivilegeInfo privilegeInfo = new PrivilegeInfo();
			privilegeInfo.setHasPrivilege(hasPrivilege);
			privilegeInfo.setCode(cur.getCode());
			privilegeInfo.setCreationTime(cur.getCreationTime());
			privilegeInfo.setGroupId(cur.getGroupId());
			privilegeInfo.setId(cur.getId());
			privilegeInfo.setName(cur.getName());
			privilegeInfo.setParentId(cur.getParentId());
			privilegeInfo.setUpdateTime(cur.getUpdateTime());
			privilegeInfo.setDescription(cur.getDescription());
			privilegeInfo.setWeight(cur.getWeight());
			privilegeInfo.setExt1(cur.getExt1());
			privilegeInfo.setExt2(cur.getExt2());
			privilegeInfo.setExt3(cur.getExt3());
			privilegeInfo.setExt4(cur.getExt4());
			privilegeInfo.setExt5(cur.getExt5());

			privilegeInfoList.add(privilegeInfo);

		}

		return privilegeInfoList;
	}

	@ApiOperation(value = "查询权限组")
	@GetMapping("getPrivilegeGroupList")
	public List<PrivilegeGroupBean> getPrivilegeGroupList() {
		Sort sort = new Sort(Sort.Direction.ASC, "id");
		List<PrivilegeGroupEntity> list = privilegeGroupRepo.findAll(sort);

		List<PrivilegeGroupBean> ret = Lists.newArrayList();

		for (PrivilegeGroupEntity cur : list) {
			PrivilegeGroupBean bean = new PrivilegeGroupBean();
			bean.setId(cur.getId());
			bean.setAppId(cur.getAppId());
			AppEntity app = appRepo.findOne(cur.getAppId());
			if (null != app) {
				bean.setAppName(app.getName());
			}
			bean.setCode(cur.getCode());
			bean.setName(cur.getName());

			ret.add(bean);
		}

		return ret;
	}

	@ApiOperation(value = "查询权限树")
	@GetMapping("getPrivilegeTree/{groupId}")
	public EleTreeNode getPrivilegeTree(@PathVariable Long groupId) {
		PrivilegeGroupEntity group = privilegeGroupRepo.findOne(groupId);
		List<PrivilegeEntity> privilegeList = privilegeRepo
				.findAllByGroupIdOrderByWeightDesc(groupId);
		EleTreeNode rootNode = new EleTreeNode(BasicConsts.ROOT_PRIVILEGE_ID, group.getName());
		TreeUtil.convert2OneEleTreeNode(rootNode, privilegeList);

		return rootNode;
	}

	@ApiOperation(value = "查询权限ID集合")
	@GetMapping("getPrivilegeIdList/{rootOrgId}/{roleId}")
	public Set<Long> getPrivilegeTree(@PathVariable Long rootOrgId, @PathVariable Long roleId) {
		List<RolePrivilegeRelationEntity> rolePrivRelationList = rolePrivilegeRelationRepo
				.findAllByRoleIdAndRootOrgId(roleId, rootOrgId);

		Set<Long> pIdSet = Sets.newHashSet();
		for (RolePrivilegeRelationEntity cur : rolePrivRelationList) {
			pIdSet.add(cur.getPrivilegeId());
		}

		return pIdSet;
	}

	@ApiOperation(value = "更新角色权限关联")
	@PostMapping("updateRolePrivilegeRelations")
	public void updateRolePrivilegeRelations(@RequestBody UpdateRolePrivilegeRelationsReq req) {
		Long rootOrgId = Long.parseLong(req.getRootOrgId());
		Long roleId = Long.parseLong(req.getRoleId());
		Long privilegeGroupId = Long.parseLong(req.getPrivilegeGroupId());
		Set<Long> privilegeIdSet = req.getPrivilegeIdSet();

		User accessUser = getAccessUser();
		Long userRootOrgId = accessUser.getRootOrgId();
		if ((!rootOrgId.equals(userRootOrgId)) && (!isSuperAdmin())) {
			throw new StatusException("B-012001", "非法请求");
		}

		rolePrivilegeService.updateRolePrivilegeRelations(rootOrgId, roleId, privilegeGroupId,
				privilegeIdSet);
	}

	@ApiOperation(value = "增加权限")
	@PostMapping("addPrivilege")
	public PrivilegeEntity addPrivilege(@RequestBody PrivilegeEntity privilege) {

		privilege.setName(privilege.getName().trim());
		privilege.setCode(privilege.getCode().trim());
		privilege.setDescription(privilege.getDescription());

		PrivilegeGroupEntity group = privilegeGroupRepo.findOne(privilege.getGroupId());
		if (null == group) {
			throw new StatusException("B-020001", "权限组不存在");
		}

		if (null == privilege.getParentId()
				|| BasicConsts.ROOT_PRIVILEGE_ID == privilege.getParentId()) {
			privilege.setParentId(null);
		} else {
			PrivilegeEntity parentPrivilege = privilegeRepo.findOne(privilege.getParentId());
			if (null == parentPrivilege) {
				throw new StatusException("B-020002", "父权限不存在");
			}
			if (!parentPrivilege.getGroupId().equals(privilege.getGroupId())) {
				throw new StatusException("B-020003", "权限组错误");
			}
		}

		PrivilegeEntity ret = privilegeRepo.save(privilege);

		return ret;
	}

	@ApiOperation(value = "更新权限")
	@PutMapping("updatePrivilege")
	public PrivilegeEntity updatePrivilege(@RequestBody PrivilegeEntity privilege) {

		privilege.setName(privilege.getName().trim());
		privilege.setCode(privilege.getCode().trim());
		privilege.setDescription(privilege.getDescription());
		PrivilegeGroupEntity group = privilegeGroupRepo.findOne(privilege.getGroupId());
		if (null == group) {
			throw new StatusException("B-020001", "权限组不存在");
		}

		if (null != privilege.getParentId()) {
			PrivilegeEntity parentPrivilege = privilegeRepo.findOne(privilege.getParentId());
			if (null == parentPrivilege) {
				throw new StatusException("B-020002", "父权限不存在");
			}
			if (!parentPrivilege.getGroupId().equals(privilege.getGroupId())) {
				throw new StatusException("B-020003", "权限组错误");
			}
		}

		PrivilegeEntity ret = privilegeRepo.save(privilege);

		return ret;
	}

	@ApiOperation(value = "删除权限")
	@DeleteMapping("deletePrivilege/{id}")
	public void deletePrivilege(@PathVariable Long id) {
		PrivilegeEntity privilege = privilegeRepo.findOne(id);
		if (null == privilege) {
			throw new StatusException("B-020004", "权限不存在");
		}

		List<PrivilegeEntity> children = privilegeRepo.findAllByParentId(privilege.getId());
		if (CollectionUtils.isNotEmpty(children)) {
			throw new StatusException("B-020005", "存在子权限");
		}
		privilegeRepo.delete(id);
	}

	@ApiOperation(value = "查询权限")
	@GetMapping("getPrivilege/{id}")
	public PrivilegeEntity getPrivilege(@PathVariable Long id) {
		PrivilegeEntity privilege = privilegeRepo.findOne(id);
		if (null == privilege) {
			throw new StatusException("B-020004", "权限不存在");
		}
		return privilege;
	}

	@ApiOperation(value = "校验权限")
	@PostMapping("checkPrivileges")
	public Map<String, Boolean> checkPrivileges(@RequestParam String privilegeCodes) {

		List<String> privilegeCodeList = RegExpUtil.findAll(privilegeCodes, "[0-9A-Za-z_]+");

		User accessUser = getAccessUser();

		List<Long> roleIdList = getAccessUserRoleIdList();

		List<PrivilegeEntity> PrivilegeEntityList = privilegeRepo
				.findAllByCodeIn(privilegeCodeList);

		Map<String, Boolean> ret = Maps.newHashMap();

		for (PrivilegeEntity cur : PrivilegeEntityList) {
			List<RolePrivilegeRelationEntity> relationEntityList = rolePrivilegeRelationRepo
					.findAllByRoleIdInAndRootOrgIdAndPrivilegeId(roleIdList,
							accessUser.getRootOrgId(), cur.getId());

			ret.put(cur.getCode(), CollectionUtils.isNotEmpty(relationEntityList));
		}

		return ret;
	}

}
