package cn.com.qmth.examcloud.core.basic.api.controller;

import cn.com.qmth.examcloud.commons.base.util.ErrorMsg;
import cn.com.qmth.examcloud.commons.web.security.entity.AccessUser;
import cn.com.qmth.examcloud.commons.web.support.ControllerSupport;
import io.swagger.annotations.ApiOperation;

import java.io.File;
import java.util.Date;
import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.PageRequest;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import cn.com.qmth.examcloud.core.basic.dao.StudentRepo;
import cn.com.qmth.examcloud.core.basic.dao.UserRepo;
import cn.com.qmth.examcloud.core.basic.dao.entity.Student;
import cn.com.qmth.examcloud.core.basic.dao.entity.UserEntity;
import cn.com.qmth.examcloud.core.basic.dao.enums.LoginType;
import cn.com.qmth.examcloud.core.basic.service.bean.UserInfo;
import cn.com.qmth.examcloud.core.basic.service.impl.StudentService;

import javax.servlet.http.HttpServletRequest;

/**
 * 学生服务API
 * Created by songyue on 17/1/14.
 */
@RestController
@RequestMapping("${app.api.root}/student")
public class StudentController extends ControllerSupport{

    @Autowired
    StudentRepo studentRepo;

    @Autowired
    StudentService studentService;

    @Autowired
    UserRepo userRepo;

    @ApiOperation(value="查询所有学生",notes="分页")
    @GetMapping("/{curPage}/{pageSize}")
    public ResponseEntity getAllStudent(HttpServletRequest request,
                                        @ModelAttribute Student studentCriteria,
                                        @PathVariable Integer curPage,
                                        @PathVariable Integer pageSize){
        cn.com.qmth.examcloud.commons.web.security.bean.User accessUser = getAccessUser();
        if(accessUser != null){
            if(accessUser.getRootOrgId().longValue() == accessUser.getOrgId().longValue()){
                studentCriteria.setRootOrgId(accessUser.getRootOrgId());
            }else{
                studentCriteria.setOrgId(accessUser.getOrgId());
            }
        }
        return new ResponseEntity(studentService.getAllStudent(studentCriteria,new PageRequest(curPage,pageSize)),HttpStatus.OK);
    }

    @ApiOperation(value="查询所有学生",notes="不分页")
    @GetMapping
    public ResponseEntity getAllStudent(HttpServletRequest request,
                                        @ModelAttribute Student studentCriteria){
        cn.com.qmth.examcloud.commons.web.security.bean.User accessUser = getAccessUser();
        if(accessUser != null){
            if(accessUser.getRootOrgId().longValue() == accessUser.getOrgId().longValue()){
                studentCriteria.setRootOrgId(accessUser.getRootOrgId());
            }else{
                studentCriteria.setOrgId(accessUser.getOrgId());
            }
        }
        return new ResponseEntity(studentService.getAllStudent(studentCriteria),HttpStatus.OK);
    }

    @ApiOperation(value="按ID查询学生",notes="ID查询")
    @GetMapping("/{id}")
    public ResponseEntity getStudentById(@PathVariable Long id){
        return new ResponseEntity(studentRepo.findOne(id),HttpStatus.OK);
    }

    @ApiOperation(value="新增学生",notes="新增")
    @PostMapping
    public ResponseEntity addStudent(@RequestBody Student student){
    	student.setCreateTime(new Date());
        return new ResponseEntity(studentService.save(student),HttpStatus.CREATED);
    }

    @ApiOperation(value="更新学生",notes="更新")
    @PutMapping
    public ResponseEntity updateStudent(@RequestBody Student student){
        return new ResponseEntity(studentService.update(student),HttpStatus.OK);
    }

    @ApiOperation(value="按ID删除学生",notes="删除")
    @DeleteMapping("/{id}")
    public ResponseEntity deleteStudent(@PathVariable Long id){
        studentRepo.delete(id);
        return new ResponseEntity(HttpStatus.OK);
    }
    
    @ApiOperation(value="学生照片绑定",notes = "照片绑定")
    @PostMapping("/photo")
    public ResponseEntity photopBundled (@RequestParam Long orgId,@RequestParam File file){
    	try {
    		List<ErrorMsg> errorMsgs = studentService.photopBundled(orgId,file);
    		return new ResponseEntity(errorMsgs,HttpStatus.OK);
		} catch (Exception e) {
			e.printStackTrace();
			return new ResponseEntity(HttpStatus.INTERNAL_SERVER_ERROR);
		}
    }
    
    @ApiOperation(value="学生登录",notes="学生登录")
    @PostMapping("/login")
    public ResponseEntity login(@RequestParam String orgCode,
                                @RequestParam String loginName,
                                @RequestParam String password,
                                @RequestParam LoginType loginType){
        try{
            UserInfo userInfo = studentService.login(orgCode,loginName,password,loginType);
            return new ResponseEntity(userInfo,HttpStatus.OK);
        }catch (Exception e){
            return new ResponseEntity(new ErrorMsg(e.getMessage()), HttpStatus.INTERNAL_SERVER_ERROR);
        }
    }
}
