package cn.com.qmth.examcloud.core.basic.api.controller;

import cn.com.qmth.examcloud.core.basic.dao.UserRepo;
import cn.com.qmth.examcloud.core.basic.dao.UserRoleRepo;
import cn.com.qmth.examcloud.core.basic.dao.entity.UserEntity;
import cn.com.qmth.examcloud.core.basic.dao.entity.UserRole;
import cn.com.qmth.examcloud.core.basic.service.impl.UserRoleService;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;

/**
 * 用户角色服务API
 * Created by songyue on 17/1/13.
 */
@RestController
@RequestMapping("${app.api.root}/userRole")
public class UserRoleController {

    @Autowired
    UserRoleService userRoleService;

    @Autowired
    UserRoleRepo userRoleRepo;

    @Autowired
    UserRepo userRepo;

    @ApiOperation(value="获取全部角色",notes="获取全部角色")
    @GetMapping("/all")
    public ResponseEntity getAllRoles(){
        return new ResponseEntity(userRoleService.findAllRoles(), HttpStatus.OK);
    }

    @ApiOperation(value="获取评卷角色",notes="获取评卷角色")
    @GetMapping("/marker")
    public ResponseEntity getMarkerRoles(){
        return new ResponseEntity(userRoleService.findMarkerRoles(), HttpStatus.OK);
    }

    @ApiOperation(value="获取非评卷角色",notes="获取非评卷角色")
    @GetMapping("/nonMarker")
    public ResponseEntity getNonMarkerRoles(){
        return new ResponseEntity(userRoleService.findNonMarkerRoles(), HttpStatus.OK);
    }

    @ApiOperation(value="获取管理员角色",notes="获取管理员角色")
    @GetMapping("/admin")
    public ResponseEntity getAdminRoles(){
        return new ResponseEntity(userRoleService.findAdminRoles(), HttpStatus.OK);
    }

    @ApiOperation(value="新增用户角色",notes="新增角色")
    @PostMapping
    public ResponseEntity addUserRole(@RequestBody UserRole userRole){
        return new ResponseEntity(userRoleRepo.save(userRole), HttpStatus.CREATED);
    }

    @ApiOperation(value="更新用户角色",notes="更新角色")
    @PutMapping
    public ResponseEntity updateUserRole(@RequestBody UserRole userRole){
        return new ResponseEntity(userRoleRepo.save(userRole), HttpStatus.OK);
    }

    @ApiOperation(value="按id删除用户",notes="删除")
    @DeleteMapping("/{id}")
    public ResponseEntity deleteUserRole(@PathVariable Long id){
        userRoleRepo.delete(id);
        return new ResponseEntity(HttpStatus.OK);
    }
    
    @ApiOperation(value="获取印刷阅卷角色",notes="获取印刷阅卷角色")
    @GetMapping("/qmther")
    public ResponseEntity getPrinterRoles(){
        return new ResponseEntity(userRoleService.findQmthRoles(), HttpStatus.OK);
    }
    
    @ApiOperation(value="获取学校机构角色",notes="获取学校机构角色")
    @GetMapping("/school")
    public ResponseEntity<Object> getSchoolRoles(){
    	return new ResponseEntity<Object>(userRoleService.findSchoolRoles(),HttpStatus.OK);
    }
}
