package cn.com.qmth.examcloud.core.basic.service.impl;
import static org.springframework.data.domain.ExampleMatcher.GenericPropertyMatchers.contains;

import java.io.InputStream;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import javax.persistence.criteria.Predicate;

import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Example;
import org.springframework.data.domain.ExampleMatcher;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.Pageable;
import org.springframework.data.jpa.domain.Specification;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import cn.com.qmth.examcloud.commons.base.util.excel.ExcelError;
import cn.com.qmth.examcloud.commons.base.util.excel.ExcelReader;
import cn.com.qmth.examcloud.commons.base.util.excel.ExcelReaderHandle;
import cn.com.qmth.examcloud.core.basic.dao.ExamSiteRepo;
import cn.com.qmth.examcloud.core.basic.dao.OrgMemRepo;
import cn.com.qmth.examcloud.core.basic.dao.OrgRepo;
import cn.com.qmth.examcloud.core.basic.dao.UserRepo;
import cn.com.qmth.examcloud.core.basic.dao.constants.Consts;
import cn.com.qmth.examcloud.core.basic.dao.entity.ExamSite;
import cn.com.qmth.examcloud.core.basic.dao.entity.Org;
import cn.com.qmth.examcloud.core.basic.dao.entity.UserEntity;
import cn.com.qmth.examcloud.core.basic.dao.entity.UserRole;
import cn.com.qmth.examcloud.core.basic.dao.enums.OrgType;
import cn.com.qmth.examcloud.core.basic.dao.enums.UserScope;
import cn.com.qmth.examcloud.core.basic.dao.enums.UserType;
import cn.com.qmth.examcloud.core.basic.service.bean.OrgDto;

@Service
public class OrgService {

    @Autowired
    OrgRepo orgRepo;

    @Autowired
    UserRepo userRepo;

	@Autowired
	ExamSiteRepo examSiteRepo;

	@Autowired
	DataSendService dataSendService;

	@Autowired
	OrgMemRepo orgMemRepo;

    @Transactional
	public List<ExcelError> importLearnCenter(Long orgId, InputStream inputStream) {
		ExcelReader excelReader = new ExcelReader(OrgDto.class);
        List<ExcelError> excelErrors = excelReader.reader(inputStream, new ExcelReaderHandle() {
            @Override
            public ExcelError handle(Object obj) {
				OrgDto dto = (OrgDto) obj;
				dto.setParentId(orgId);
				dto.setRootId(orgId);
                ExcelError error = importCheck(dto);
                if (error == null) {
                	saveOrgAndExamSite(dto);
                }
                return error;
            }
        });
		return excelErrors;
	}

	/**
	 * 保存学习中心和考点
	 * @param orgDto
     */
	private void saveOrgAndExamSite(OrgDto orgDto) {
		Org org = orgRepo.findFirstByParentIdAndCode(orgDto.getParentId(), orgDto.getCode());
		if (org == null) {

			Org tempOrg = orgRepo.save(orgAssembler(orgDto));
			orgDto.setId(tempOrg.getId());
			examSiteRepo.save(examSiteAssembler(orgDto));
			createOrgUser(tempOrg);

		} else {

			org.setName(orgDto.getName());
			org.setContacts(orgDto.getContacts());
			org.setTelphone(orgDto.getTelphone());
			orgRepo.save(org);

			orgDto.setId(org.getId());
			ExamSite examSite = examSiteRepo.findFirstByOrgIdAndCode(org.getId(), orgDto.getExamSiteCode());
			if (examSite == null) {
				examSiteRepo.save(examSiteAssembler(orgDto));
			} else {
				examSite.setName(orgDto.getExamSiteName());
//				examSite.setContacts(orgDto.getContacts());
//				examSite.setTelphone(orgDto.getTelphone());
				examSiteRepo.save(examSite);
			}
		}

	}

	/**
	 * 创建学习中心用户
	 * @param tempOrg
     */
	private void createOrgUser(Org tempOrg){
		UserEntity user = new UserEntity(tempOrg.getName(), UserScope.ORG, tempOrg.getRootId(), tempOrg.getId(), UserType.NOT_STUDENT);
		user.setLoginName(tempOrg.getCode());
		user.setEnable(true);
		user.setCreateTime(new Date());
		user.setPassword(Consts.DEFAULT_PASSWORD);
		List<UserRole> userRoles = new ArrayList<UserRole>();
		userRoles.add(new UserRole("LC_USER"));
		user.setUserRoles(userRoles);
		userRepo.save(user);
	}

	private Org orgAssembler(OrgDto orgDto){
		Org org = new Org();
		org.setRootId(orgDto.getRootId());
		org.setParentId(orgDto.getParentId());
		org.setCode(orgDto.getCode());
		org.setName(orgDto.getName());
		org.setContacts(orgDto.getContacts());
		org.setTelphone(orgDto.getTelphone());
		org.setCreateTime(new Date());
		org.setEnable(true);
		//添加机构类型
		org.setType(OrgType.SCHOOL);
		return org;
	}

	private ExamSite examSiteAssembler(OrgDto orgDto){
		ExamSite examSite = new ExamSite();
		examSite.setCode(orgDto.getExamSiteCode());
		examSite.setName(orgDto.getExamSiteName());
		examSite.setOrgId(orgDto.getId());
//		examSite.setContacts(orgDto.getContacts());
//		examSite.setTelphone(orgDto.getTelphone());
		examSite.setCreateTime(new Date());
		examSite.setEnable(true);
		return examSite;
	}

	private ExcelError importCheck(OrgDto dto) {
		if(StringUtils.isBlank(dto.getCode())){
			return new ExcelError("中心代码不能为空");
		}
		if(StringUtils.isBlank(dto.getName())){
			return new ExcelError("中心名称不能为空");
		}
		if(StringUtils.isBlank(dto.getExamSiteCode())){
			return new ExcelError("考点代码不能为空");
		}
		if(StringUtils.isBlank(dto.getExamSiteName())){
			return new ExcelError("考点名称不能为空");
		}
		if(StringUtils.isBlank(dto.getContacts())){
			return new ExcelError("联系人不能为空");
		}
		if(StringUtils.isBlank(dto.getTelphone())){
			return new ExcelError("联系电话不能为空");
		}
		return null;
	}

	/**
	 * 启用/禁用机构；禁用机构时禁用该机构下的所有用户
	 * @param id
	 * @param enable
	 * @return
	 */
	@Transactional
	public Org enableSchool(Long id, boolean enable) {
		Org org = orgRepo.findOne(id);
		org.setEnable(enable);
		orgRepo.save(org);
		if(enable == false){
			List<UserEntity> users = userRepo.findByOrgId(id);
			for (UserEntity user : users) {
				user.setEnable(false);
				userRepo.save(user);
			}
		}
		return org;
	}

	public Page<Org> findAll(Org orgCriteria, Pageable pageable) {
		if(orgCriteria.getType() == null){
			Specification<Org> specification = getSpecification(orgCriteria);
			return orgRepo.findAll(specification,pageable);
		}
		//orgCriteria.setType(OrgType.SCHOOL);
		ExampleMatcher exampleMatcher = ExampleMatcher.matching()
				.withMatcher("name", contains())
				.withMatcher("code", contains());
		Example<Org> examExamStudentple = Example.of(orgCriteria,
				exampleMatcher);
		return orgRepo.findAll(examExamStudentple, pageable);
	}
	
	private Specification<Org> getSpecification(Org orgCriteria) {
		Specification<Org> specification = (root, query, cb) -> {
		    List<Predicate> predicates = new ArrayList<>();
		    if(!StringUtils.isEmpty(orgCriteria.getName())){
		    	predicates.add(cb.equal(root.get("name"),orgCriteria.getName()));
		    }
		    if(!StringUtils.isEmpty(orgCriteria.getCode())){
		    	predicates.add(cb.equal(root.get("code"),orgCriteria.getCode()));
		    }
		    predicates.add(cb.notEqual(root.get("type"), OrgType.SCHOOL));
		    predicates.add(cb.equal(root.get("parentId"),orgCriteria.getParentId()));
		    return cb.and(predicates.toArray(new Predicate[predicates.size()]));
		};
		return specification;
	}

	public List<Org> findAll(Org orgCriteria) {
		ExampleMatcher exampleMatcher = ExampleMatcher.matching()
				.withMatcher("name", contains())
				.withMatcher("code", contains());
        Example<Org> examExample = Example.of(orgCriteria,exampleMatcher);
        return orgRepo.findAll(examExample);
	}

	public Org save(Org org) throws Exception{
		checkCode(org.getParentId(),org.getCode());
		org.setCreateTime(new Date());
		Org reOrg = orgRepo.save(org);
		dataSendService.sendOrg(reOrg);
		//删除缓存
		orgMemRepo.remove(reOrg.getId());
		return reOrg;
	}

	private void checkCode(Long parentId,String code) {
		Org old = orgRepo.findFirstByParentIdAndCode(parentId,code);
		if(old!=null){
			throw new RuntimeException("代码已存在");
		}
	}

	public Org update(Long id, Org org) {
		Org old = orgRepo.findOne(id);
		if(!old.getCode().equals(org.getCode())){
			checkCode(org.getParentId(),org.getCode());
		}
		org.setUpdateTime(new Date());
		Org reOrg = orgRepo.save(org);
		dataSendService.sendOrg(reOrg);
		//删除缓存
		orgMemRepo.remove(reOrg.getId());
		return reOrg;
	}

	public Org findOne(Long id){
		Org cacheOrg = orgMemRepo.get(id);
		if(cacheOrg != null){
			return cacheOrg;
		}else{
			Org org = orgRepo.findOne(id);
			if(org != null){
				orgMemRepo.set(id,org);
			}
			return org;
		}
	}
	
	public Org findFirstByParentIdAndCode(Long parentId,String code){
		return orgRepo.findFirstByParentIdAndCode(parentId, code);
	}
	
}
